@@ -62,8 +62,6 @@ terminal_fprintf (FILE* fp,
   }
 }
 
-#if GLIB_CHECK_VERSION (2, 50, 0)
-
 /* Need to install a special log writer so we never output
  * anything without the '# ' prepended, in case --print-environment
  * is used.
@@ -82,8 +80,6 @@ terminal_log_writer (GLogLevelFlags log_level,
   return G_LOG_WRITER_HANDLED;
 }
 
-#endif /* GLIB 2.50 */
-
 static GOptionContext *get_goption_context (TerminalOptions *options);
 
 static TerminalSettingsList *
@@ -993,7 +989,11 @@ terminal_options_parse (int *argcp,
   options->execute = FALSE;
 
   const char *startup_id = g_getenv ("DESKTOP_STARTUP_ID");
-  options->startup_id = g_strdup (startup_id && startup_id[0] ? startup_id : NULL);
+  if (startup_id && startup_id[0] &&
+      g_utf8_validate (startup_id, -1, NULL))
+    options->startup_id = g_strdup (startup_id);
+  else
+    options->startup_id = NULL;
   options->display_name = NULL;
   options->initial_windows = NULL;
   options->default_role = NULL;
@@ -1098,6 +1098,7 @@ terminal_options_parse (int *argcp,
     return FALSE;
   }
 
+  options->wait = wait != 0;
   return options;
 }
 
