@@ -65,14 +65,37 @@ receiver_child_exited_cb (TerminalReceiver *receiver,
     g_main_loop_quit (data->loop);
 }
 
+static void
+factory_name_owner_notify_cb (TerminalFactory *factory,
+                              GParamSpec *pspec,
+                              RunData *data)
+{
+  /* Name owner change to NULL can only mean that the server
+   * went away before it could send out our child-exited signal.
+   * Assume the server was killed and thus our child process
+   * too, and return with the corresponding exit code.
+   */
+  if (g_dbus_proxy_get_name_owner(G_DBUS_PROXY (factory)) != NULL)
+    return;
+
+  data->status = W_EXITCODE(0, SIGKILL);
+
+  if (g_main_loop_is_running (data->loop))
+    g_main_loop_quit (data->loop);
+}
+
 static int
-run_receiver (TerminalReceiver *receiver)
+run_receiver (TerminalFactory *factory,
+              TerminalReceiver *receiver)
 {
   RunData data = { g_main_loop_new (NULL, FALSE), 0 };
-  gulong id = g_signal_connect (receiver, "child-exited",
-                                G_CALLBACK (receiver_child_exited_cb), &data);
+  gulong receiver_exited_id = g_signal_connect (receiver, "child-exited",
+                                                G_CALLBACK (receiver_child_exited_cb), &data);
+  gulong factory_notify_id = g_signal_connect (factory, "notify::g-name-owner",
+                                               G_CALLBACK (factory_name_owner_notify_cb), &data);
   g_main_loop_run (data.loop);
-  g_signal_handler_disconnect (receiver, id);
+  g_signal_handler_disconnect (receiver, receiver_exited_id);
+  g_signal_handler_disconnect (factory, factory_notify_id);
   g_main_loop_unref (data.loop);
 
   /* Mangle the exit status */
@@ -197,6 +220,7 @@ handle_exec_error (const char *service_name,
 static gboolean
 factory_proxy_new_for_service_name (const char *service_name,
                                     gboolean ping_server,
+                                    gboolean connect_signals,
                                     TerminalFactory **factory_ptr,
                                     char **service_name_ptr,
                                     GError **error)
@@ -208,7 +232,7 @@ factory_proxy_new_for_service_name (const char *service_name,
   gs_unref_object TerminalFactory *factory =
     terminal_factory_proxy_new_for_bus_sync (G_BUS_TYPE_SESSION,
                                              G_DBUS_PROXY_FLAGS_DO_NOT_LOAD_PROPERTIES |
-                                             G_DBUS_PROXY_FLAGS_DO_NOT_CONNECT_SIGNALS,
+                                             connect_signals ? 0 : G_DBUS_PROXY_FLAGS_DO_NOT_CONNECT_SIGNALS,
                                              service_name,
                                              TERMINAL_FACTORY_OBJECT_PATH,
                                              NULL /* cancellable */,
@@ -268,6 +292,7 @@ factory_proxy_new (TerminalOptions *options,
     gs_free_error GError *err = NULL;
     if (factory_proxy_new_for_service_name (options->server_unique_name,
                                             TRUE,
+                                            options->wait,
                                             factory_ptr,
                                             service_name_ptr,
                                             &err)) {
@@ -287,13 +312,15 @@ factory_proxy_new (TerminalOptions *options,
 
   return factory_proxy_new_for_service_name (service_name,
                                              FALSE,
+                                             options->wait,
                                              factory_ptr,
                                              service_name_ptr,
                                              error);
 }
 
 static void
-handle_show_preferences (const char *service_name)
+handle_show_preferences (TerminalOptions *options,
+                         const char *service_name)
 {
   gs_free_error GError *error = NULL;
   gs_unref_object GDBusConnection *bus = NULL;
@@ -324,6 +351,9 @@ handle_show_preferences (const char *service_name)
   g_variant_builder_open (&builder, G_VARIANT_TYPE ("av"));
   g_variant_builder_close (&builder);
   g_variant_builder_open (&builder, G_VARIANT_TYPE ("a{sv}"));
+  if (options->startup_id)
+    g_variant_builder_add (&builder, "{sv}",
+                           "desktop-startup-id", g_variant_new_string (options->startup_id));
   g_variant_builder_close (&builder);
 
   if (!g_dbus_connection_call_sync (bus,
@@ -369,7 +399,7 @@ handle_options (TerminalOptions *options,
   g_get_charset (&encoding);
 
   if (options->show_preferences) {
-    handle_show_preferences (service_name);
+    handle_show_preferences (options, service_name);
   } else {
     /* Make sure we open at least one window */
     terminal_options_ensure_window (options);
@@ -520,11 +550,9 @@ main (int argc, char **argv)
 {
   int exit_code = EXIT_FAILURE;
 
-#if GLIB_CHECK_VERSION (2, 50, 0)
   g_log_set_writer_func (terminal_log_writer, NULL, NULL);
-#endif
 
-  g_set_prgname ("gterminalclient");
+  g_set_prgname ("gnome-terminal");
 
   setlocale (LC_ALL, "");
 
@@ -532,13 +560,6 @@ main (int argc, char **argv)
 
   _terminal_debug_init ();
 
-  /* Make a NULL-terminated copy since we may need it later */
-  gs_free char **argv_copy = g_new (char *, argc + 1);
-  int i;
-  for (i = 0; i < argc; ++i)
-    argv_copy [i] = argv [i];
-  argv_copy [i] = NULL;
-
   gs_free_error GError *error = NULL;
   gs_free_options TerminalOptions *options = terminal_options_parse (&argc, &argv, &error);
   if (options == NULL) {
@@ -571,7 +592,7 @@ main (int argc, char **argv)
     return exit_code;
 
   if (receiver != NULL) {
-    exit_code = run_receiver (receiver);
+    exit_code = run_receiver (factory, receiver);
     g_object_unref (receiver);
   } else
     exit_code = EXIT_SUCCESS;
